'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _ = require('lodash');
var TypeValidator = require('./validators/TypeValidator');
var PresenceValidator = require('./validators/PresenceValidator');
var InclusionValidator = require('./validators/InclusionValidator');

var DataValidator = function () {
  function DataValidator(model, data) {
    _classCallCheck(this, DataValidator);

    this.data = data;
    this.model = model;
    this.errors = null;
  }

  _createClass(DataValidator, [{
    key: 'call',
    value: function call() {
      var _this = this;

      this._validateMetadata();
      this._attributes().forEach(function (attribute) {
        var type = _this._type(attribute);
        if (type === undefined) throw new Error('Missing type for attribute \'' + attribute + '\'');else if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) !== 'object') _this._validate(attribute);else {
          _this._validate(attribute);
          var values = _this.data[attribute];
          var composedValidationFailed = _this.errors && _this.errors[attribute];
          if (!composedValidationFailed && values) {
            if (!Array.isArray(values)) values = [values];
            values.forEach(function (value) {
              var errors = new DataValidator(type, value).call();
              if (errors) _this._addError(attribute, 'is not valid');
            });
          }
        }
      });
      return this.errors;
    }
  }, {
    key: '_validate',
    value: function _validate(attribute) {
      var _this2 = this;

      var values = this.data[attribute];
      if (!Array.isArray(values)) values = [values];
      values.forEach(function (value) {
        _this2._validatePresence(attribute, value);
        _this2._validateType(attribute, value);
        _this2._validateInclusion(attribute, value);
      });
    }
  }, {
    key: '_validatePresence',
    value: function _validatePresence(attribute, value) {
      var nullable = this._nullable(attribute);
      var isValid = PresenceValidator.call(value, nullable);
      if (!isValid) this._addError(attribute, 'must be given');
    }
  }, {
    key: '_validateType',
    value: function _validateType(attribute, value) {
      var type = this._type(attribute);
      if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) === 'object') return;
      var validator = TypeValidator.typeValidatorFor(type);
      var validType = validator(value, this.model[attribute]);
      if (!validType) this._addError(attribute, 'is not valid');
    }
  }, {
    key: '_validateInclusion',
    value: function _validateInclusion(attribute, value) {
      var list = this._inclusion(attribute);
      var isIncluded = InclusionValidator.call(value, list);
      if (!isIncluded) this._addError(attribute, 'is not included in the list');
    }
  }, {
    key: '_validateMetadata',
    value: function _validateMetadata() {
      var metadata = this._metadata();
      if (metadata.rejectUnknownFields) this._validateUnknownFields();
    }
  }, {
    key: '_validateUnknownFields',
    value: function _validateUnknownFields() {
      var givenAttributes = Object.keys(this.data);
      var expectedAttributes = this._attributes();
      if (givenAttributes.every(function (attribute) {
        return _.includes(expectedAttributes, attribute);
      })) return;
      this._addError('model', 'has unknown attributes');
    }
  }, {
    key: '_type',
    value: function _type(attribute) {
      return this.model[attribute].type;
    }
  }, {
    key: '_inclusion',
    value: function _inclusion(attribute) {
      return this.model[attribute].inclusion;
    }
  }, {
    key: '_nullable',
    value: function _nullable(attribute) {
      return this.model[attribute].nullable;
    }
  }, {
    key: '_metadata',
    value: function _metadata() {
      return this.model.metadata || {};
    }
  }, {
    key: '_attributes',
    value: function _attributes() {
      return Object.keys(_.omit(this.model, ['metadata']));
    }
  }, {
    key: '_addError',
    value: function _addError(attribute, message) {
      var errors = this.errors || {};
      errors[attribute] = (errors[attribute] || []).concat(message);
      this.errors = errors;
    }
  }]);

  return DataValidator;
}();

module.exports = DataValidator;