'use strict';

var _ = require('lodash');
var moment = require('moment');
var DataTypes = require('../DataTypes');

var TypeValidator = {
  typeValidatorFor: function typeValidatorFor(type) {
    var validators = {};
    validators[DataTypes.DATE] = this.isDate;
    validators[DataTypes.STRING] = this.isString;
    validators[DataTypes.BOOLEAN] = this.isBoolean;
    validators[DataTypes.INTEGER] = this.isInteger;
    var validator = validators[type];
    if (validator === undefined) throw new Error('Cannot pick validator for undefined type');
    return validator;
  },
  isString: function isString(data) {
    var rules = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (TypeValidator._isNullOrUndefined(data)) return true;
    return _.isString(data);
  },
  isBoolean: function isBoolean(data) {
    var rules = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (TypeValidator._isNullOrUndefined(data)) return true;
    return _.isBoolean(data);
  },
  isInteger: function isInteger(data) {
    var rules = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (TypeValidator._isNullOrUndefined(data)) return true;
    return _.isInteger(data, rules);
  },
  isDate: function isDate(data) {
    var rules = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (TypeValidator._isNullOrUndefined(data)) return true;
    var dateFormat = rules.dateFormat;
    if (!dateFormat) return moment(data).isValid();
    if (!TypeValidator.isString(data, rules) || !TypeValidator.isString(dateFormat)) return false;
    var date = moment(data, dateFormat);
    if (!date.isValid()) return false;else return date.format(dateFormat) === data;
  },
  _isNullOrUndefined: function _isNullOrUndefined(data) {
    return _.isNull(data) || _.isUndefined(data);
  }
};

module.exports = TypeValidator;